﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Interfaces;

namespace Common
{
    public class LogManager:ILog
    {
        protected class LogFileInfo
        {
            public FileStream fsLogFile = null; //文件流
            public DateTime dtLastAction = DateTime.Now - TimeSpan.FromDays(1); //最后写的时间
        }

        public LogManager(string logPath)
        {
            if (logPath.LastIndexOf('\\') != logPath.Length - 1)
                logPath += "\\";
            if (!Directory.Exists(logPath))
                Directory.CreateDirectory(logPath);
            strLogPath = logPath;
            bInitialized = true;
        }

        protected void Close()
        {
            foreach (LogFileInfo lfi in dictLogFiles.Values)
            {
                if (lfi.fsLogFile != null)
                {
                    lfi.fsLogFile.Close();
                    lfi.fsLogFile = null;
                }
            }
        }

        public void Log(LogType logType, string moduleName, string content, params object[] values)
        {
            LogFileInfo lfi;
            if (dictLogFiles.ContainsKey(logType.ToString()))
                lfi = dictLogFiles[logType.ToString()];
            else
            {
                lfi = new LogFileInfo();
                dictLogFiles.Add(logType.ToString(), lfi);
            }

            DateTime dtLogFile = DateTime.Now;
            if (dtLogFile.Day != lfi.dtLastAction.Day)
            {
                //如果文件流已经打开，就关闭它
                if (lfi.fsLogFile != null)
                    lfi.fsLogFile.Close();

                //如果目录不存在，创建之
                string logSubFolder = LogHelper.GetLogSubFolder(strLogPath, dtLogFile);
                if (!Directory.Exists(logSubFolder))
                    Directory.CreateDirectory(logSubFolder);

                //如果Log文件已存在，就以追加方式打开它
                //如果Log文件不存在，就创建它
                string strLogFile = LogHelper.GetLogFileName(strLogPath, logType.ToString(), dtLogFile);
                if (File.Exists(strLogFile))
                {
                    lfi.fsLogFile = File.Open(strLogFile, FileMode.Append, FileAccess.Write, FileShare.Read);
                }
                else
                {
                    lfi.fsLogFile = File.Open(strLogFile, FileMode.Create, FileAccess.Write, FileShare.Read);
                    byte[] bom = Encoding.UTF8.GetPreamble();
                    lfi.fsLogFile.Write(bom, 0, bom.Length);
                }
            }

            //写内容
            StringBuilder sb = new StringBuilder();
            sb.Append(LogHelper.DtToString(DateTime.Now));
            sb.Append(" ");
            sb.Append(moduleName);
            sb.Append(" ");
            if (values.Any())
                sb.AppendFormat(content, values);
            else
                sb.Append(content);
            sb.Replace("\r\n", "\t");
            sb.Replace('\r', '\t');
            sb.Replace('\n', '\t');
            sb.Append("\r\n");
            byte[] byToWrite = Encoding.UTF8.GetBytes(sb.ToString());
            lock (thisLock)
            {
                lfi.fsLogFile.Write(byToWrite, 0, byToWrite.Length);
                lfi.fsLogFile.Flush();
            }

            lfi.dtLastAction = DateTime.Now;
        }


        protected Dictionary<string, LogFileInfo> dictLogFiles = new Dictionary<string, LogFileInfo>();
        //可帮助用LogType来获到LogFileInfo

        protected string strLogPath; //Log根目录，末尾带上反斜杠
        protected bool bInitialized = false; //是否已经初始化
        protected static bool bOneInstanceCreated = false; //已创建一个实例
        protected readonly Object thisLock = new Object();
    }
}
